import 'package:grpc/grpc.dart';
import 'package:grpc_communication/src/agent.pb.dart';
import 'package:grpc_communication/src/agent.pbgrpc.dart';

class GrpcAgentCommunication {
  late ClientChannel _channel;
  late AgentControlClient _client;

  GrpcAgentCommunication(String host, int port) {
    _channel = ClientChannel(
      host,
      port: port,
      options: const ChannelOptions(credentials: ChannelCredentials.insecure()),
    );
    _client = AgentControlClient(_channel);
  }

  Stream<Agent> streamAgents() {
    return _client.streamAgents(Empty()).asBroadcastStream();
  }

  Future<CreateAgentResponse> createAgent(String pacAgentId, String ipAddressExpected) async {
    try {
      final request = CreateAgentRequest()
        ..pacAgentId = pacAgentId
        ..ipAddressExpected = ipAddressExpected;

      return await _client.createAgent(request);
    } catch (e) {
      print('Caught error in createAgent: $e');
      rethrow;
    }
  }

  Future<List<Agent>> fetchAgents() async {
    try {
      final response = await _client.listAgents(ListAgentsRequest());
      return response.agents;
    } catch (e) {
      print('Caught error in fetchAgents: $e');
      rethrow;
    }
  }


  void dispose() {
    _channel.shutdown();
  }
}