import 'package:grpc/grpc.dart';
import 'package:grpc_communication/src/resourcegroup.pb.dart';
import 'package:grpc_communication/src/resourcegroup.pbgrpc.dart';

class GrpcResourceCommunication {
  late ClientChannel _channel;
  late DataCenterControlClient _client;

  GrpcResourceCommunication(String host, int port) {
    _channel = ClientChannel(
      host,
      port: port,
      options: const ChannelOptions(credentials: ChannelCredentials.insecure()),
    );
    _client = DataCenterControlClient(_channel);
  }

  Stream<ResourceGroup> streamResourceGroups() {
    return _client.streamResourceGroups(Empty()).asBroadcastStream();
  }

  Future<CreateGroupResponse> createResourceGroup(String groupName, String location) async {
    try {
      final request = CreateGroupRequest()
        ..groupName = groupName
        ..location = location;

      return await _client.createGroup(request);
    } catch (e) {
      print('Caught error in createGroup: $e');
      rethrow;
    }
  }

  Future<List<ResourceGroup>> fetchResourceGroups() async {
    try {
      final response = await _client.listResourceGroups(ListResourceGroupsRequest());
      return response.resourceGroups;
    } catch (e) {
      print('Caught error in createGroup: $e');
      rethrow;
    }
  }


  void dispose() {
    _channel.shutdown();
  }
}